public inbox for gdb-cvs@sourceware.org
help / color / mirror / Atom feed
* [binutils-gdb] gdb/darwin: skip over WIFSTOPPED wait4 status
@ 2022-02-24 15:49 Simon Marchi
  0 siblings, 0 replies; only message in thread
From: Simon Marchi @ 2022-02-24 15:49 UTC (permalink / raw)
  To: gdb-cvs

https://sourceware.org/git/gitweb.cgi?p=binutils-gdb.git;h=7ff917016a203cdff3074abfcf96c1553944af94

commit 7ff917016a203cdff3074abfcf96c1553944af94
Author: Dominique Quatravaux <dominique.quatravaux@epfl.ch>
Date:   Thu Feb 24 09:23:21 2022 -0500

    gdb/darwin: skip over WIFSTOPPED wait4 status
    
    On modern Darwin's, there appears to be a new circumstance in which a
    MACH_NOTIFY_DEAD_NAME message can be received, and which was not
    previously accounted for: to signal the WIFSTOPPED condition in the
    debuggee. In that case the debuggee is not dead yet (and in fact,
    counting it as dead would cause a zombie leak - A process in such a
    state reparents to PID 1, but cannot be killed).
    
     - Read and ignore such messages (counting on the next exception message
       to let us know of the inferior's new state again)
     - Refactor logging so as to clearly distinguish between the
       MACH_NOTIFY_DEAD_NAME cases (WIFEXITED, WIFSTOPPED, signal, or
       something else), and warn in the last case
    
    Co-authored-by: Louis-He <1726110778@qq.com>
    Co-authored-by: Philippe Blain <levraiphilippeblain@gmail.com>
    Change-Id: Ie86904a894e9bd154e6b674b1bfbfbaee7fde3e1

Diff:
---
 gdb/darwin-nat.c | 31 +++++++++++++++++++++++++------
 1 file changed, 25 insertions(+), 6 deletions(-)

diff --git a/gdb/darwin-nat.c b/gdb/darwin-nat.c
index 8b0ecfd5b77..adbe7474c68 100644
--- a/gdb/darwin-nat.c
+++ b/gdb/darwin-nat.c
@@ -1063,7 +1063,7 @@ darwin_nat_target::decode_message (mach_msg_header_t *hdr,
     }
   else if (hdr->msgh_id == 0x48)
     {
       /* MACH_NOTIFY_DEAD_NAME: notification for exit.  */
       int res;
 
       res = darwin_decode_notify_message (hdr, &inf);
@@ -1103,15 +1103,34 @@ darwin_nat_target::decode_message (mach_msg_header_t *hdr,
 		  return minus_one_ptid;
 		}
 	      if (WIFEXITED (wstatus))
-		status->set_exited (WEXITSTATUS (wstatus));
-	      else
+		{
+		  status->set_exited (WEXITSTATUS (wstatus));
+	          inferior_debug (4, _("darwin_wait: pid=%d exit, status=0x%x\n"),
+				  res_pid, wstatus);
+		}
+	      else if (WIFSTOPPED (wstatus))
+		{
+		  /* Ignore stopped state, it will be handled by the next
+		     exception.  */
+		  status->set_ignore ();
+		  inferior_debug (4, _("darwin_wait: pid %d received WIFSTOPPED\n"),
+				  res_pid);
+		  return minus_one_ptid;
+		}
+	      else if (WIFSIGNALED (wstatus))
 		{
 		  status->set_signalled
 		    (gdb_signal_from_host (WTERMSIG (wstatus)));
+		  inferior_debug (4, _("darwin_wait: pid=%d received signal %d\n"),
+				  res_pid, status->sig());
+		}
+	      else
+		{
+		  status->set_ignore ();
+		  warning (_("Unexpected wait status after MACH_NOTIFY_DEAD_NAME "
+		             "notification: 0x%x"), wstatus);
+		  return minus_one_ptid;
 		}
-
-	      inferior_debug (4, _("darwin_wait: pid=%d exit, status=0x%x\n"),
-			      res_pid, wstatus);
 
 	      return ptid_t (inf->pid);
 	    }


^ permalink raw reply	[flat|nested] only message in thread

only message in thread, other threads:[~2022-02-24 15:49 UTC | newest]

Thread overview: (only message) (download: mbox.gz / follow: Atom feed)
-- links below jump to the message on this page --
@@ -1063,7 +1063,7 @@ darwin_nat_target::decode_message (mach_msg_header_t *hdr,
   else if (hdr->msgh_id == 0x48)
     {
-      /* MACH_NOTIFY_DEAD_NAME: notification for exit.  */
+      /* MACH_NOTIFY_DEAD_NAME: notification for exit *or* WIFSTOPPED.  */
       int res;
 
2022-02-24 15:49 [binutils-gdb] gdb/darwin: skip over WIFSTOPPED wait4 status Simon Marchi

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for read-only IMAP folder(s) and NNTP newsgroup(s).